/*
  Tool            EditFileList
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   TSE v4 upwards, all variants
  Version         v1   16 Nov 2023

  This tool executes TSE's standard EditFile() command, and then enhances its
  lists with the <Alt E> key to edit the path\filenames of such a list in a new
  buffer.

  No, other file properties were intentionally not copied to the edit buffer.

  Yes, this also works for lists generated with the "-s" parameter.

  Yes, this has everything to do with the fact that the "-s" option no longer
  crashes TSE as of TSE v4.50 rc 13.


  INSTALLATION

  Copy this file to TSE's "mac" directorory, and compile it there,
  for example by opening it there in TSE and applying the Macro Compile menu.

  To run the tool execute it as a macro,
  for example by applying the Macro Execute menu and entering "EditFileList".


  CAVEAT

  In my experience dd 16 Nov 2023 EditFile("-s <some file specification>")
  still aborts TSE with a segmentation fault in Linux for some file
  specifications.
  When EditFile() does not abort in Linux, then this tool works there too.


  HISTORY

  v1   16 Nov 2023
    Initial release.

*/





// Start of compatibility restrictions and mitigations

#ifdef LINUX
  #define WIN32 FALSE
#endif

#ifndef WIN32
   16-bit versions of TSE are not supported. You need at least TSE 4.0.
#endif

#ifndef EDITOR_VERSION
   Editor Version is older than TSE 3.0. You need at least TSE 4.0.
#endif

#if EDITOR_VERSION < 4000h
   Editor Version is older than TSE 4.0. You need at least TSE 4.0.
#endif



/*
  From TSE versions newer than v4.4 onwards the edit states that
  QueryEditState() returns are available as names instead of values.

  The below bit of code allows you to use the names on the left
  in at least any version from TSE v4.0 onwards.
*/
#if EDITOR_VERSION > 4400h
  #define STATE_TWOKEY            _STATE_TWOKEY_
  #define STATE_WARN              _STATE_WARN_
  #define STATE_EDITOR_PAUSED     _STATE_EDITOR_PAUSED_
  #define STATE_POPWINDOW         _STATE_POPWINDOW_
  #define STATE_PROCESS_IN_WINDOW _STATE_PROCESS_IN_WINDOW_
  #define STATE_MENU              _STATE_MENU_
  #define STATE_PROMPTED          _STATE_PROMPTED_
#else
  #define STATE_TWOKEY            0x0200
  #define STATE_WARN              0x0400
  #define STATE_EDITOR_PAUSED     0x0800
  #define STATE_POPWINDOW         0x1000
  #define STATE_PROCESS_IN_WINDOW 0x2000
  #define STATE_MENU              0x4000
  #define STATE_PROMPTED          0x3000
#endif

// End of compatibility restrictions and mitigations





// Global constants and semi-constants
string MACRO_NAME [MAXSTRINGLEN] = ''

string LIST_FOOTER_FULL [78] =
  'Alt E-Edit List  Esc-Cancel  Ctrl Space-Tag  Enter-Select  F10-Menu  Alt <--Up'

string LIST_FOOTER_FULL_BUT_COMPRESSED [73] =
  'Alt E-Edit List Esc-Cancel Ctrl Space-Tag Enter-Select F10-Menu Alt <--Up'

string LIST_FOOTER_CUT [67] =
  'Alt E-Edit List  Esc-Cancel  Ctrl Space-Tag  Enter-Select  F10-Menu'

string LIST_FOOTER_CUT_AND_COMPRESSED [63] =
  'Alt E-Edit List Esc-Cancel Ctrl Space-Tag Enter-Select F10-Menu'





#ifdef LINUX
  #define META_DATA_END_COLUMN 21
#else
  #if EDITOR_VERSION == 4000h
    #define META_DATA_END_COLUMN 13
  #else
    #if EDITOR_VERSION == 4200h
      #define META_DATA_END_COLUMN 13
    #else
      #if EDITOR_VERSION == 4400h
        #define META_DATA_END_COLUMN 14
      #else
        #define META_DATA_END_COLUMN 17
      #endif
    #endif
  #endif
#endif



// Global variables
integer lst_id = FALSE
integer tmp_id = FALSE


proc idle()
  UnHook(idle)
  if lst_id
    GotoBufferId(lst_id)
    UpdateDisplay(_ALL_WINDOWS_REFRESH_)
  endif
  PurgeMacro(CurrMacroFilename())
end idle


proc after_getkey()
  if  QueryEditState() == (STATE_POPWINDOW | STATE_PROCESS_IN_WINDOW)
  and Query(Key)       == <Alt e>
    PushBlock()
    PushLocation()
    MarkLine(1, NumLines())
    lst_id = NewFile()
    tmp_id = GetBufferId(MACRO_NAME + ':Raw_List')
    if tmp_id
      GotoBufferId(tmp_id)
      EmptyBuffer()
    else
      tmp_id = CreateTempBuffer()
      ChangeCurrFilename(MACRO_NAME + ':Raw_List',
                         _DONT_PROMPT_|_DONT_EXPAND_|_OVERWRITE_)
    endif
    CopyBlock()
    MarkLine(1, NumLines())
    GotoBufferId(lst_id)
    CopyBlock()
    BinaryMode(TRUE)
    MarkColumn(1, 1, NumLines(), META_DATA_END_COLUMN)
    KillBlock()
    BinaryMode(FALSE)
    lReplace(Chr(0) + '.@', '', 'gnx')

    BegFile()
    PopLocation()
    PopBlock()
    PushKey(<Escape>)
    PushKey(<Escape>)
  endif
end after_getkey


proc hilite_footer_part(string list_footer, integer list_footer_x, string part)
  if Pos(part, list_footer)
    PutStrAttrXY( list_footer_x + Pos(part, list_footer) - 1,
                  Query(WindowY1) + Query(WindowRows),
                  list_footer[Pos(part, list_footer):Length(part)],
                  '',
                  Query(MenuTextLtrAttr))
  endif
end hilite_footer_part


proc nonedit_idle()
  string  list_footer [MAXSTRINGLEN] = ''
  integer list_footer_x              = 0

  UnHook(nonedit_idle)

  if     Query(WindowCols) >= Length(LIST_FOOTER_FULL)
    list_footer = LIST_FOOTER_FULL
  elseif Query(WindowCols) >= Length(LIST_FOOTER_FULL_BUT_COMPRESSED)
    list_footer = LIST_FOOTER_FULL_BUT_COMPRESSED
  elseif Query(WindowCols) >= Length(LIST_FOOTER_CUT)
    list_footer = LIST_FOOTER_CUT
  else
    list_footer = LIST_FOOTER_CUT_AND_COMPRESSED
  endif

  list_footer_x = iif(Length(list_footer) >= Query(WindowCols),
                      Query(WindowX1),
                      Query(WindowX1) + (Query(WindowCols) - Length(list_footer)) / 2)

  PutStrAttrXY( list_footer_x,
                Query(WindowY1) + Query(WindowRows),
                list_footer,
                '',
                Query(MenuTextAttr))

  hilite_footer_part(list_footer, list_footer_x, 'Alt E')
  hilite_footer_part(list_footer, list_footer_x, 'Esc')
  hilite_footer_part(list_footer, list_footer_x, 'Ctrl Space')
  hilite_footer_part(list_footer, list_footer_x, 'Enter')
  hilite_footer_part(list_footer, list_footer_x, 'F10')
  hilite_footer_part(list_footer, list_footer_x, 'Alt <-')
end nonedit_idle


proc pickfile_startup()
  Hook(_NONEDIT_IDLE_, nonedit_idle)
end pickfile_startup


proc Main()
  MACRO_NAME = SplitPath(CurrMacroFilename(), _NAME_)
  Hook(_AFTER_GETKEY_, after_getkey)
  Hook(_IDLE_        , idle)
  Hook(_PICKFILE_STARTUP_, pickfile_startup)
  EditFile()
  UnHook(after_getkey)
  UnHook(pickfile_startup)
end Main


